# Estatistica no R - Aula 4 (Regressao)
# Luis Anunciacao (Psicometria, PUC-Rio/University of Oregon)
# luisfca@gmail.com www.anovabr.com

library("tidyverse") #carregar pacote

bsq_regress <- dados_brasil %>% 
  select(bsq_soma, mulheres, idade, imc, faz_esporte, familia_esporte, eat_soma) %>% 
  drop_na() #criar base propria
eat_regress <- dados_brasil %>% 
  select(eat_soma, mulheres, idade, imc, faz_esporte, familia_esporte, bsq_soma) %>% 
  drop_na() #criar base propria 

# regressao linear
rl <- lm(data=bsq_regress, bsq_soma ~. ) 
library("apaTables")
apa.reg.table(rl)
library("effects")
all_effects <- allEffects(rl)
plot(all_effects)

# Luis Anunciacao, 2017
# This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License. 
#https://creativecommons.org/licenses/by-nc-sa/4.0/


##LASSO REGRESSION (Least Absolute Shrinkage and Selection Operator)
#BSQ
x <- as.matrix(bsq_regress[,-1]) # Remover as VIs (deixar apenas as VDs)
y <- as.double(as.matrix(bsq_regress[,1])) # apenas VI
set.seed(123)
cv.lasso.bsq <- cv.glmnet(x, y, family="gaussian", type.measure="deviance", penalty.factor=c(0, rep(1, ncol(bsq_regress) - 2)), alpha=1)
#best_lambda <- cv.lasso.bsq$lambda.min #caso eu queira estipular a penalizacao pelo menor lambda
l <- predict(cv.lasso.bsq, type=c("coefficients"), newx = x[,]) #, s = best_lambda) #Reporta os coeficientes
tidy(l)

#R2
bsq_predicted <- predict(cv.lasso.bsq, newx = x) #calcular o valor previsto pelo modelo
bsq_regress <- bsq_regress %>%  mutate(bsq_previsto = bsq_predicted) #adicionar a base
bsq_regress %>% summarise(n=n(), SQT = sum((bsq_soma - mean(bsq_soma))^2), 
                          SQR = sum((bsq_soma - bsq_previsto)^2), 
                          R2 = 1-SQR/SQT,
                          EQM = sqrt(mean((bsq_soma - bsq_previsto)^2))) #computar o R2

#EAT
x <- as.matrix(eat_regress[,-1]) # Remover as VIs (deixar apenas as VDs)
y <- as.double(as.matrix(eat_regress[,1])) # apenas VI
set.seed(123)
cv.lasso.eat <- cv.glmnet(x, y, family="gaussian", type.measure="deviance", penalty.factor=c(0, rep(1, ncol(eat_regress) - 2)), alpha=1)
#best_lambda <- cv.lasso.bsq$lambda.min #caso eu queira estipular a penalizacao pelo menor lambda
l <- predict(cv.lasso.eat, type=c("coefficients"), newx = x[,]) #, s = best_lambda) #Reporta os coeficientes
tidy(l)

#R2
eat_predicted <- predict(cv.lasso.eat, newx = x) #calcular o valor previsto pelo modelo
eat_regress <- eat_regress %>%  mutate(eat_previsto = eat_predicted) #adicionar a base
eat_regress %>% summarise(SQT = sum((eat_soma - mean(eat_soma))^2), 
                          SQR = sum((eat_soma - eat_previsto)^2), 
                          R2 = 1-SQR/SQT,
                          EQM = sqrt(mean((eat_soma - eat_previsto)^2))) #computar o R2

#bsq_regress %>% summarise_each(funs(sum(is.na(.))))
#bsq_regress %>% summarise(sum(is.na(.)))

##RIDGE REGRESSION (glmnet)
#lambda <- 10^seq(10, -2, length = 100)
#bsq_regress2 <- bsq_regress
#mod.ridge <- cv.glmnet(x, y, family="gaussian", alpha=0, lambda=lambda)
#plot(mod.ridge)
#coef(mod.ridge, s=best_lambda)
#ridge.pred <- predict(mod.ridge, s = best_lambda, newx = x[,])
#bsq_regress2 <- bsq_regress2 %>% mutate(preditolasso = ridge.pred)

#Mesmos resultados
#mod.lasso <- glmnet(x, y, family="gaussian", alpha=1) #retorna o mesmo valor do cv.glmnet
#plot(mod.lasso)
#j <- predict(mod.lasso, type="coefficients", newx = x[,]) #s = best_lambda, 

#PURRR
#iv_vars <- c("mulheres", "idade", "imc","faz_esporte","familia_esporte","eat_soma")
#make_model <- function(nm) lm(bsq_regress[c("bsq_soma", nm)])
#fits <- Map(make_model, iv_vars)
#glance_tidy <- function(x) c(unlist(glance(x)), unlist(tidy(x)[, -1]))
#results <- iv_vars %>% Map(f = make_model) %>% sapply(glance_tidy)
#Stepwise regression
#step(lm(bsq_soma ~. , data=bsq_regress),direction = "both", trace = 1) %>% summary

